<?php

// $Id: input_formats.api.inc,v 1.1 2010/06/05 01:42:10 dagmar Exp $

/**
 * Enables and removes input formats from code.
 */
function input_formats_api_syncronize() {
  $result = db_query("SELECT * FROM {input_formats}");
  while ($ob = db_fetch_object($result)) {
    $loaded_input_formats[$ob->name] = $ob->name;
  }
  $default_input_formats = module_invoke_all('input_formats_default');

  /* @TODO: Should remove filters is owner module is disabled?
  // Check what input formats must be deleted
  // Delete all non overriden input formats that were disabled from code
  foreach ($loaded_input_formats as $loaded_input_format) {
    if (!$loaded_input_format->overriden) {
      $to_delete = input_formats_api_load($loaded_input_format->name);
      input_formats_api_delete($to_delete);
    }
  }*/

  // Check what input formats must be enabled
  // Enable all the input formats provided by
  // defaults that are not enabled yet
  foreach ($default_input_formats as $default_input_format) {
    if (!isset($loaded_input_formats[$default_input_format['machine_name']])) {
      input_formats_api_save($default_input_format);
    }
  }
}

/**
 * Create the machine names for the existents input formats
 */
function input_formats_api_sync_names() {
  db_query("TRUNCATE {input_formats}");
  $res = db_query("SELECT format, name FROM {filter_formats}");
  while ($filter = db_fetch_object($res)) {
    $machine_name = str_replace(' ', '_', strtolower($filter->name));
    db_query("INSERT INTO {input_formats} (format, name, overriden) VALUES (%d, '%s', 0)", $filter->format, $machine_name);
  }
}

/**
 * Load an input format using a format id.
 */
function input_formats_api_load_by_id($format, $full_load = TRUE) {
  $input_format_name = input_formats_api_get_machine_name($format);
  return input_formats_api_load($input_format_name, $full_load, $format);
}

/**
 * Load an input format.
 *
 * @param $input_format_name
 *   A string containing the machine name of the input format.
 * @param $full_load
 *   Optional. If TRUE, the input format will include all the settings.
 *   Useful for export tasks.
 * @param $id
 *   Optional. Represent the numeric ID of the input format.
 *
 * @return
 *   An input format array or NULL if something failed.
 */
function input_formats_api_load($input_format_name, $full_load = TRUE, $id = NULL) {
  if (!$id) {
    $input_format_id = input_formats_api_get_id($input_format_name);
    if (!$input_format_id) {
      return;
    }
  }
  else {
    $input_format_id = $id;
  }

  $format = filter_formats($input_format_id);

  $input_format = array(
    'id' => $format->format,
    'machine_name' => $input_format_name,
    'name' => check_plain($format->name),
    'roles' => array(),
    'order' => array(),
    'settings' => array(),
    'extras' => array(),
  );

  if (!$full_load) {
    return $input_format;
  }

  $roles = array();
  $format_roles = array_filter(explode(',', $format->roles));
  $user_roles = input_formats_api_user_roles();

  foreach ($format_roles as $rid) {
    $roles[$rid] = $user_roles[$rid];
  }
  $input_format['roles'] = $roles;

  $list_format = filter_list_format($input_format_id);
  foreach ($list_format as $filter) {
    if (!isset($input_format['order'])) {
      $input_format['order'][$filter->module] = array();
    }
    $input_format['order'][$filter->module][$filter->delta] = array(
      '#description' => check_plain($filter->name),
      '#value' => $filter->weight,
    );
  }

  $variables = array();
  $list = filter_list_format($input_format_id);
  foreach ($list as $filter) {

    $form_module = module_invoke($filter->module, 'filter', 'settings', $filter->delta, $input_format_id);
    if (isset($form_module) && is_array($form_module)) {
      $module_variables = array();

      _input_formats_api_walk_form($form_module, $module_variables);
      $variables[$filter->module] = $module_variables;
    }
  }

  if (count($variables)) {
    // Trim the _1, _2, _3 that filters uses in {variables}
    $trim = -1 * (strlen($input_format_id) + 1);
    foreach ($variables as $module_name => $module_variables) {
      foreach ($module_variables as $variable_name => $title) {
        $independent_variable = substr($variable_name, 0, $trim);
        $input_format['settings'][$module_name][$independent_variable] = array(
          '#description' => check_plain($title),
          '#value' => variable_get($variable_name, ''),
        );
      }
    }
  }
  // Allow other modules to include options to the input format.
  drupal_alter('input_format', $input_format);
  return $input_format;
}

/**
 * Check if a input format array is well formed.
 *
 * @param $input_format
 *   An array that represents an input format
 */
function input_formats_api_check_integrity($input_format) {
  $errors = FALSE;
  $keys = array('machine_name', 'name', 'roles', 'order', 'settings', 'extras');
  foreach ($keys as $key) {
    $errors |= $errors || !isset($input_format[$key]);
  }
  $errors = $errors || !is_array($input_format['roles']);
  $errors = $errors || !is_array($input_format['order']);
  $errors = $errors || !is_array($input_format['settings']);
  $errors = $errors || !is_array($input_format['extras']);
  return !$errors;
}

/**
 * Obtains the numeric id for a given input format.
 */
function input_formats_api_get_id($input_format_name) {
  return db_result(db_query("SELECT format FROM {input_formats} WHERE name = '%s'", $input_format_name));
}

/**
 * Obtains the numeric id for a given input format.
 */
function input_formats_api_get_machine_name($format) {
  return db_result(db_query("SELECT name FROM {input_formats} WHERE format = %d", $format));
}


/**
 * Check if a input format exists.
 *
 * @param $input_format_name
 *   A string containing the machine name of the input format.
 */
function input_formats_api_exist($input_format_name) {
  return (bool)input_formats_api_get_id($input_format_name);
}

function input_formats_api_overriden($input_format) {
  return (bool)db_result(db_query("SELECT overriden FROM {input_formats} WHERE name = '%s'"), $input_format['machine_name']);
}

function input_formats_api_check_human_name($name) {
  $name = trim($name);
  $result = db_fetch_object(db_query("SELECT format FROM {filter_formats} WHERE name='%s'", $name));
  if ($result) {
    form_set_error('name', t('Filter format names need to be unique. A format named %name already exists.', array('%name' => $name)));
  }
  return $result;
}

/**
 * Load a definition of an input format from hooks_input_formats_default().
 */
function input_formats_api_load_from_code($input_format) {
  $default_input_formats = module_invoke_all('input_formats_default');
  if (isset($default_input_formats[$input_format['machine_name']])) {
    return $default_input_formats[$input_format['machine_name']];
  }
}

/**
 * Ever time a new filter is saved, a new input format is created with a
 * machine name.
 */
function input_formats_api_new($format, $machine_name) {
  // Ensure the machine name.
  $machine_name = str_replace(' ', '_', strtolower($machine_name));
  db_query("INSERT INTO {input_formats} (format, name, overriden) VALUES (%d, '%s', 0)", $format, $machine_name);
}

/**
 * Save a input format object it into a available input format.
 *
 * @param $input_format
 *   An array that represents an input format
 */
function input_formats_api_save($input_format) {
  if (!input_formats_api_check_integrity($input_format)) {
    drupal_set_message(t('The input format is corrupted and cannot be imported'), 'error');
    return;
  }

  if (input_formats_api_exist($input_format['machine_name'])) {
    drupal_set_message(t("There is already an Input Format with the machine name: @name.
    The input format cannot be imported.", array('@name' => $input_format['machine_name'])), 'error');
    return;
  }

  if (input_formats_api_check_human_name($input_format['name'])) {
    return;
  }

  db_query("INSERT INTO {filter_formats} (name) VALUES ('%s')", $input_format['name']);
  $format = $input_format['id'] = db_result(db_query("SELECT MAX(format) AS format FROM {filter_formats}"));

  db_query("DELETE FROM {filters} WHERE format = %d", $input_format['id']);
  foreach ($input_format['order'] as $module => $data) {
    foreach($data as $delta => $weight) {
      db_query("INSERT INTO {filters} (format, module, delta, weight) VALUES (%d, '%s', %d, %d)", $format, $module, $delta, $weight);

      $cache &= !module_invoke($module, 'filter', 'no cache', $delta);
    }
  }

  // Check that roles are exactly equals (in name and rid).
  // If a role doens't match. Don't grant access to this input format
  // to that role.
  // Call to our user_roles function to avoid language issues.
  $user_roles = input_formats_api_user_roles();

  $roles = array();
  foreach ($input_format['roles'] as $rid => $role_name) {
    if (isset($user_roles[$rid]) && $user_roles[$rid] == $role_name) {
      $roles[] = $rid;
    }
    else {
      drupal_set_message(t("Warning! Input Filter: @name is trying to set grant access
         to users with role: @role (role id: @rid), but you don't have any role with that
         name. The access to @name wasn't granted to role: @role.",
         array('@name' => $input_format['name'], '@role' => $role_name, '@rid' => $rid)), 'error');
    }
  }
  $roles = ','. implode(',', $roles) .',';
  db_query("UPDATE {filter_formats} SET cache = %d, name='%s', roles = '%s' WHERE format = %d", $cache, $input_format['name'], $roles, $format);

  // Save settings for this input format
  foreach ($input_format['settings'] as $module_name => $module_settings) {
    foreach ($module_settings as $variable_name => $variable_value) {
      variable_set($variable_name . '_' . $format, $variable_value);
    }
  }

  cache_clear_all($format .':', 'cache_filter', TRUE);

  input_formats_api_new($format, $input_format['machine_name']);
  drupal_set_message(t('A new Input format was created. Its recomended to !link its configuration settings.', array('!link' => l(t('check'), 'admin/settings/filters/' . $format))));

  // Inform to other modules that a new input format was enabled
  module_invoke_all('input_format_enabled', $input_format);
}

/**
 * Check if two input formats are different.
 *
 * Usefull to check if a input format is overriden.
 *
 * @param $from_code
 *   An input format array.
 * @param $from_db
 *   An input format array.
 * @update_db
 *   If TRUE, when $form_code != $from_db, the value overriden
 *   from {input_formats} is updated.
 */
function input_formats_api_compare($from_code, $from_db, $update_db = FALSE) {
  $export_from_code = input_formats_api_export($from_code);
  $export_from_db = input_formats_api_export($from_code);
  $equals = $export_from_code == $export_from_db;
  if (!$equals && $update_db && $from_code['machine_name'] == $from_db['machine_name']) {
    db_query("UPDATE {input_formats} SET overriden = 1 WHERE name = '%s'", $from_db['machine_name']);
  }
}

/**
 * Render a input format as plain text.
 */
function input_formats_api_export($input_format, $prefix = '') {
  $code = array();
  $code[] = $prefix . "array(";
  $code[] = $prefix . "  'name' => '" . $input_format['name'] . "',";
  $code[] = $prefix . "  'machine_name' => '" . $input_format['machine_name'] . "',";
  $code[] = $prefix . "  'roles' => " . input_formats_api_var_export($input_format['roles'], $prefix . "  ") . ",";

  $code[] = $prefix . "";

  $code[] = $prefix . "  'order' => array(";
  foreach ($input_format['order'] as $module_name => $module_settings) {
    $code[] = $prefix . "    '$module_name' => array(";
    foreach ($module_settings as $variable_name => $data) {
      $code[] = $prefix . "      // (Delta => Order) for: " . $data['#description'];
      $code[] = $prefix . "      '$variable_name' => " . input_formats_api_var_export($data['#value'], $prefix . "      ") . ",";
    }
    $code[] = $prefix . "    ),";
  }
  $code[] = $prefix . "  ),";

  $code[] = $prefix . "";

  $code[] = $prefix . "  'settings' => array(";
  foreach ($input_format['settings'] as $module_name => $module_settings) {
    $code[] = $prefix . "    //Settings for: $module_name";
    $code[] = $prefix . "    '$module_name' => array(";
    foreach ($module_settings as $variable_name => $data) {
      $code[] = $prefix . "      // " . $data['#description'];
      $code[] = $prefix . "      '$variable_name' => " . input_formats_api_var_export($data['#value'], $prefix . "      ") . ",";
    }
    $code[] = $prefix . "    ),";
  }
  $code[] = $prefix . "  ),";

  $code[] = $prefix . "";

  $code[] = $prefix . "  'extras' => array(";
  foreach ($input_format['extras'] as $module_name => $module_settings) {
    $code[] = $prefix . "    //Settings for: $module_name";
    $code[] = $prefix . "    '$module_name' => array(";
    foreach ($module_settings as $variable_name => $data) {
      $code[] = $prefix . "      // " . $data['#description'];
      $code[] = $prefix . "      '$variable_name' => " . input_formats_api_var_export($data['#value'], "      ") . ",";
    }
    $code[] = $prefix . "    ),";
  }
  $code[] = $prefix . "  ),";
  $code[] = $prefix . ");";

  return implode("\n", $code);
}

/**
 * Delete an input format from the database.
 *
 * @param $input_format
 *   An array that represents an input format
 */
function input_formats_api_delete($input_format) {
  // Filtred HTML cannot be deleted
  if ($input_format['id'] == 1) {
    return;
  }
  db_query("DELETE FROM {input_formats} WHERE name = '%s'", $input_format['machine_name']);

  $form_state = array(
    'values' => array(
      'format' => $input_format['id'],
    ),
  );

  // Copied from filter_admin_delete_submit()
  db_query("DELETE FROM {filter_formats} WHERE format = %d", $form_state['values']['format']);
  db_query("DELETE FROM {filters} WHERE format = %d", $form_state['values']['format']);

  $default = variable_get('filter_default_format', 1);
  // Replace existing instances of the deleted format with the default format.
  db_query("UPDATE {node_revisions} SET format = %d WHERE format = %d", $default, $form_state['values']['format']);
  db_query("UPDATE {comments} SET format = %d WHERE format = %d", $default, $form_state['values']['format']);
  db_query("UPDATE {boxes} SET format = %d WHERE format = %d", $default, $form_state['values']['format']);

  cache_clear_all($form_state['values']['format'] .':', 'cache_filter', TRUE);
  module_invoke_all('input_format_deleted', $to_delete);
}

/**
 * Retrives a list of modules that an input format need to work.
 *
 * @param $input_format
 *   An array that represents an input format
 */
function input_formats_api_dependencies($input_format) {
  $dependencies = array();
  $dependencies += array_keys($input_format['order']);
  $dependencies += array_keys($input_format['extras']);
  return $dependencies;
}

/**
 * Helper functions
 */

/**
 * Walk each form items searching for non fieldsets widgets.
 *
 * This widgets contains the names of the variables that the input
 * format uses.
 */
function _input_formats_api_walk_form($form, &$variables) {
  foreach (element_children($form) as $key) {
    if ($form[$key]['#type'] == 'fieldset') {
      _input_formats_api_walk_form($form[$key], $variables);
    }
    else {
      $variables[$key] = $form[$key]['#title'];
    }
  }
}

/**
 * Export a variable.
 *
 * This is similar to ctools_var_export, but putting this function here
 * avoid a dependency with ctools.
 */
function input_formats_api_var_export($var, $prefix = '  ') {
  if (is_array($var)) {
    if (empty($var)) {
      $output = 'array()';
    }
    else {
      $output = "array(\n";
      foreach ($var as $key => $value) {
        $output .= $prefix . "'$key' => " . input_formats_api_var_export($value, $prefix) . ",\n";
      }
      $output .= ')';
    }
  }
  else if (is_object($var) && get_class($var) === 'stdClass') {
    // var_export() will export stdClass objects using an undefined
    // magic method __set_state() leaving the export broken. This
    // workaround avoids this by casting the object as an array for
    // export and casting it back to an object when evaluated.
    $output .= '(object) ' . input_formats_api_var_export((array) $var);
  }
  else if (is_bool($var)) {
    $output = $var ? 'TRUE' : 'FALSE';
  }
  else {
    $output = var_export($var, TRUE);
  }

  if ($prefix) {
    $output = str_replace("\n", "\n$prefix", $output);
  }

  return $output;
}

/**
 * Returns the user roles without translate role names.
 */
function input_formats_api_user_roles() {
  $user_roles = array();
  $result = db_query("SELECT rid, name FROM {role}");
  while ($role = db_fetch_object($result)) {
    $user_roles[$role->rid] = $role->name;
  }
  return $user_roles;
}
